@@ -1,8 +1,8 @@
 #ifndef ABOUT_H
 #define ABOUT_H
 /*
-    roxterm - GTK+ 2.0 terminal emulator with tabs
-    Copyright (C) 2004 Tony Houghton <h@realh.co.uk>
+    roxterm - VTE/GTK terminal emulator with tabs
+    Copyright (C) 2004-2015 Tony Houghton <h@realh.co.uk>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,12 +24,10 @@
 #include "defns.h"
 #endif
 
-/* parent isn't vital; it just allows the use of gtk_show_about_dialog, but
- * that isn't actually used in the current implementation.
- */
 void about_dialog_show(GtkWindow *parent,
-		GtkAboutDialogActivateLinkFunc www_hook,
-		GtkAboutDialogActivateLinkFunc email_hook, gpointer hook_data);
+        gboolean (*uri_handler)(GtkAboutDialog *, char *, gpointer),
+        gpointer hook_data
+);
 
 #endif /* ABOUT_H */
 
