@@ -99,6 +99,7 @@ struct MultiWin {
     gboolean show_add_tab_button;
     GdkGeometry geom_hints;
     GdkWindowHints geom_hint_mask;
+    gboolean has_geometry;
 };
 
 static double multi_win_zoom_factors[] = {
@@ -308,10 +309,17 @@ MultiTab *multi_tab_new_defer_connect(MultiWin * parent,
 MultiTab *multi_tab_new(MultiWin * parent, gpointer user_data_template)
 {
     MultiTab *tab = multi_tab_new_defer_connect(parent, user_data_template);
+    //g_debug("multi_tab_new connecting signals for %p", tab->user_data);
     multi_tab_connect_misc_signals(tab->user_data);
     return tab;
 }
 
+/* For setting a breakpoint */
+static void multi_tab_free(MultiTab *tab)
+{
+    g_free(tab);
+}
+
 static void multi_tab_delete_without_notifying_parent(MultiTab * tab,
         gboolean destroy_widgets)
 {
@@ -324,6 +332,7 @@ static void multi_tab_delete_without_notifying_parent(MultiTab * tab,
     {
         tab->postponed_free = FALSE;
     }
+    //g_debug("Deleting tab with roxterm %p", tab->user_data);
     (*multi_tab_destructor) (tab->user_data);
     tab->user_data = NULL;
 
@@ -349,7 +358,7 @@ static void multi_tab_delete_without_notifying_parent(MultiTab * tab,
         tab->menu_bar_item = NULL;
     }
     if (!tab->postponed_free)
-        g_free(tab);
+        multi_tab_free(tab);
 }
 
 void multi_tab_delete(MultiTab * tab)
@@ -950,7 +959,7 @@ static gboolean multi_win_state_event_handler(GtkWidget *widget,
         {
             gtk_window_set_geometry_hints(GTK_WINDOW(widget), NULL, NULL, 0);
         }
-        else
+        else if (win->has_geometry)
         {
             multi_win_apply_geometry_hints(win);
         }
@@ -1333,7 +1342,7 @@ static void multi_win_name_tab_action(MultiWin * win)
     }
     tab->rename_dialog = NULL;
     if (tab->postponed_free)
-        g_free(tab);
+        multi_tab_free(tab);
 }
 static void multi_win_save_session_action(MultiWin * win)
 {
@@ -1457,20 +1466,33 @@ static void multi_win_set_window_title_action(MultiWin * win)
     }
 }
 
-static void multi_win_next_tab_action(MultiWin * win)
+void multi_win_next_tab(MultiWin * win, gboolean wrap)
 {
-    if (win->wrap_switch_tab && multi_win_at_last_tab(win))
-        gtk_notebook_set_current_page(GTK_NOTEBOOK(win->notebook), 0);
+    if (multi_win_at_last_tab(win))
+    {
+        if (wrap)
+            gtk_notebook_set_current_page(GTK_NOTEBOOK(win->notebook), 0);
+    }
     else
+    {
         gtk_notebook_next_page(GTK_NOTEBOOK(win->notebook));
+    }
 }
 
-static void multi_win_previous_tab_action(MultiWin * win)
+void multi_win_next_tab_action(MultiWin * win)
 {
-    if (win->wrap_switch_tab && multi_win_at_first_tab(win))
+    multi_win_next_tab(win, win->wrap_switch_tab);
+}
+
+void multi_win_previous_tab_action(MultiWin * win)
+{
+    if (multi_win_at_first_tab(win))
     {
-        gtk_notebook_set_current_page(GTK_NOTEBOOK(win->notebook),
-                win->ntabs - 1);
+        if (win->wrap_switch_tab)
+        {
+            gtk_notebook_set_current_page(GTK_NOTEBOOK(win->notebook),
+                    win->ntabs - 1);
+        }
     }
     else
     {
@@ -1826,10 +1848,32 @@ static gboolean multi_win_delete_event_cb(GtkWidget *widget, GdkEvent *event,
         MultiWin *win)
 {
     if (!multi_win_delete_handler(widget, event, win))
+    {
         multi_win_destructor(win, TRUE);
+    }
     return TRUE;
 }
 
+static void multi_win_size_allocate(GtkWidget *widget, GdkRectangle *alloc,
+        MultiWin *win)
+{
+    (void) widget;
+    (void) alloc;
+    if (!win->has_geometry)
+    {
+        MultiTab *tab = win->current_tab;
+        if (!tab)
+        {
+            g_warning("Size allocated to window with no current tab");
+            return;
+        }
+        int columns, rows, width, height;
+        (*multi_win_size_func)(tab->user_data,
+                FALSE, &columns, &rows);
+        multi_win_process_geometry(win, tab, columns, rows, &width, &height);
+    }
+}
+
 MultiWin *multi_win_new_blank(Options *shortcuts,
         int zoom_index,
         gboolean disable_menu_shortcuts, gboolean disable_tab_shortcuts,
@@ -1880,6 +1924,8 @@ MultiWin *multi_win_new_blank(Options *shortcuts,
     }
     g_signal_connect(win->gtkwin, "draw",
             G_CALLBACK(multi_win_draw_chrome_bg), win);
+    g_signal_connect(win->gtkwin, "size-allocate",
+            G_CALLBACK(multi_win_size_allocate), win);
 
     multi_win_set_colormap(win);
 
@@ -2633,11 +2679,9 @@ static gboolean multi_win_process_geometry(MultiWin *win,
     if (!tab)
         tab = win->current_tab;
     multi_win_geometry_func(tab->user_data, &geom, &hint_mask);
-    /*
     g_debug("VTE cell size %dx%d, padding %dx%d",
             geom.width_inc, geom.height_inc,
             geom.base_width, geom.base_height);
-    */
 
     /* Get difference in size between toplevel window and the geometry
      * widget.
@@ -2653,13 +2697,13 @@ static gboolean multi_win_process_geometry(MultiWin *win,
     //        gw, gh, bw, bh);
     if (gw <= 1 || gh <= 1)
     {
+        *width = 0;
+        *height = 0;
         return FALSE;
     }
-    /*
-    g_debug("Want to allocate %dx%d to terminal", 
-            columns * geom.width_inc + geom.base_width,
-            rows * geom.height_inc + geom.base_height);
-    */
+    //g_debug("Want to allocate %dx%d to terminal", 
+    //        columns * geom.width_inc + geom.base_width,
+    //        rows * geom.height_inc + geom.base_height);
 
     geom.base_width += bw - gw;
     geom.base_height += bh - gh;
@@ -2680,7 +2724,7 @@ static gboolean multi_win_process_geometry(MultiWin *win,
     geom.min_height += wh - gh;
     geom.base_width += ww - bw;
     geom.base_height += wh - bh;
-    if (geom.min_width != win->geom_hints.min_width ||
+    if (!win->has_geometry || geom.min_width != win->geom_hints.min_width ||
         geom.min_height != win->geom_hints.min_height ||
         geom.base_width != win->geom_hints.base_width ||
         geom.base_height != win->geom_hints.base_height ||
@@ -2690,6 +2734,7 @@ static gboolean multi_win_process_geometry(MultiWin *win,
         //g_debug("Updating geometry");
         win->geom_hints = geom;
         win->geom_hint_mask = hint_mask;
+        win->has_geometry = TRUE;
         return TRUE;
     }
     //g_debug("Geometry unchanged");
@@ -2704,9 +2749,19 @@ void multi_win_set_initial_geometry(MultiWin *win, const char *geom,
 
     if (multi_win_parse_geometry(geom, &columns, &rows, &x, &y, &xy))
     {
+        //g_debug("multi_win_set_initial_geometry: %d columns, %d rows",
+        //        columns, rows);
         multi_win_process_geometry(win, tab, columns, rows, &width, &height);
-        multi_win_apply_geometry_hints(win);
-        gtk_window_set_default_size(GTK_WINDOW(win->gtkwin), width, height);
+        //g_debug("processed geometry to : %dx%d", width, height);
+        if (width > 1 && height > 1)
+        {
+            multi_win_apply_geometry_hints(win);
+            gtk_window_set_default_size(GTK_WINDOW(win->gtkwin), width, height);
+        }
+        else
+        {
+            g_warning("Unable to determine geometry");
+        }
         /* Honor initial position on launch: it is indispensable and ubiquitous */
         if (xy)
         {
