@@ -1,6 +1,6 @@
 /*
-    roxterm - GTK+ 2.0 terminal emulator with tabs
-    Copyright (C) 2004 Tony Houghton <h@realh.co.uk>
+    roxterm - VTE/GTK terminal emulator with tabs
+    Copyright (C) 2004-2015 Tony Houghton <h@realh.co.uk>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,20 +24,10 @@
 #include <stdarg.h>
 #include <string.h>
 
-#ifndef HAVE_G_MKDIR_WITH_PARENTS
-#include <sys/stat.h>
-#include <sys/types.h>
-#endif
-
 #include "dlg.h"
 #include "globalopts.h"
 #include "optsfile.h"
 
-#ifndef HAVE_G_FILE_SET_CONTENTS
-extern gboolean g_file_set_contents(const gchar * filename,
-					const gchar * contents, gssize length, GError ** error);
-#endif
-
 static char **options_pathv = NULL;
 
 /* Append 2 NULL-terminated arrays of strings, avoiding duplicates; the strings
@@ -236,7 +226,7 @@ GKeyFile *options_file_open(const char *leafname, const char *group_name)
 	}
 
 	first_group = g_key_file_get_start_group(kf);
-	if (strcmp(first_group, group_name))
+	if (!first_group || strcmp(first_group, group_name))
 	{
 		dlg_critical(NULL, _("Options file %s does not start with group '%s'"),
 				filename, group_name);
@@ -258,52 +248,12 @@ gboolean options_file_mkdir_with_parents(const char *dirname)
 		g_critical(_("Invalid directory name '%s'"), dirname);
 		return FALSE;
 	}
-#ifdef HAVE_G_MKDIR_WITH_PARENTS
 	if (g_mkdir_with_parents(dirname, 0755) == -1)
 	{
 		dlg_critical(NULL, _("Failed to create directory '%s': %s"),
 			dirname, strerror(errno));
 		return FALSE;
 	}
-#else
-	if (mkdir(dirname, 0755))
-	{
-		char *parent;
-
-		switch (errno)
-		{
-			case EEXIST:
-				return TRUE;
-			case ENOENT:
-				/* Recursively try to make parent */
-				parent = g_path_get_dirname(dirname);
-				if (parent && !strcmp(parent, dirname))
-				{
-					g_critical(_("Unable to create parent directory of %s"),
-						dirname);
-					g_free(parent);
-					return FALSE;
-				}
-				if (options_file_mkdir_with_parents(parent))
-				{
-					/* Successfully made parent, try again to make target */
-					g_free(parent);
-					if (!mkdir(dirname, 0755))
-						return TRUE;
-				}
-				else
-				{
-					g_free(parent);
-				}
-				/* Fall through to default (failure) if couldn't create parent
-				 */
-			default:
-				dlg_critical(NULL, _("Failed to create directory '%s': %s"),
-					dirname, strerror(errno));
-				return FALSE;
-		}
-	}
-#endif /* HAVE_G_MKDIR_WITH_PARENTS */
 	return TRUE;
 }
 
@@ -313,7 +263,7 @@ char *options_file_filename_for_saving(const char *leafname, ...)
 	va_list ap;
 	char *result;
 	char *partial;
-	
+
 	if (!xdg_dir)
 	{
 		xdg_dir = g_build_filename(g_get_user_config_dir(),
@@ -433,4 +383,67 @@ int options_file_lookup_int_with_default(
 	return result;
 }
 
+gboolean options_file_copy_to_user_dir(GtkWindow *window,
+        const char *src_path, const char *family, const char *new_leaf)
+{
+    char *new_path = options_file_filename_for_saving(family, NULL);
+    char *buffer = NULL;
+    gsize size;
+    GError *error = NULL;
+    gboolean success = FALSE;
+
+    if (!g_file_test(new_path, G_FILE_TEST_IS_DIR))
+    {
+        if (g_mkdir_with_parents(new_path, 0755) == -1)
+        {
+            dlg_warning(window, _("Unable to create directory '%s'"), new_path);
+            return FALSE;
+        }
+    }
+    g_free(new_path);
+    new_path = options_file_filename_for_saving(family, new_leaf, NULL);
+    if (g_file_test(src_path, G_FILE_TEST_IS_REGULAR))
+    {
+        if (g_file_get_contents(src_path, &buffer, &size, &error))
+            success = g_file_set_contents(new_path, buffer, size, &error);
+        g_free(buffer);
+    }
+    else
+    {
+        const char *content = "";
+        if (!strcmp(family, "Profiles"))
+            content = "[roxterm profile]";
+        else if (!strcmp(family, "Colours"))
+            content = "[roxterm colour scheme]";
+        else if (!strcmp(family, "Shortcuts"))
+            content = "[roxterm shortcuts scheme]";
+        success = g_file_set_contents(new_path, content, -1, &error);
+    }
+    if (!success)
+    {
+        dlg_warning(window, _("Unable to copy profile/scheme: %s"),
+                error && error->message ? error->message : _("unknown reason"));
+    }
+    if (error)
+        g_error_free(error);
+    g_free(new_path);
+    return success;
+}
+
+char *options_file_make_editable(GtkWindow *window,
+        const char *family_name, const char *name)
+{
+    char *path = options_file_filename_for_saving(family_name, name, NULL);
+    if (!g_file_test(path, G_FILE_TEST_EXISTS))
+    {
+        char *src = options_file_build_filename(family_name, name, NULL);
+        if (src)
+        {
+            options_file_copy_to_user_dir(window, src, family_name, name);
+            g_free(src);
+        }
+    }
+    return path;
+}
+
 /* vi:set sw=4 ts=4 noet cindent cino= */
