@@ -279,6 +279,12 @@ void on_scrollbar_pos_toggled(GtkRadioButton *button, ProfileGUI *pg)
     profilegui_set_scrollbar_shading(pg);
 }
 
+void on_kinetic_toggled(GtkToggleButton *button, ProfileGUI *pg)
+{
+    gboolean kinetic = gtk_toggle_button_get_active(button);
+    capplet_set_boolean_toggle(&pg->capp, "pixel_scrolling", kinetic);
+}
+
 #define DEFAULT_BACKSPACE_BINDING 0
 #define DEFAULT_DELETE_BINDING 0
 
@@ -467,6 +473,7 @@ static void profilegui_fill_in_dialog(ProfileGUI * pg)
     capplet_set_boolean_toggle(&pg->capp, "always_show_tabs", TRUE);
     capplet_set_boolean_toggle(&pg->capp, "show_add_tab_btn", TRUE);
     capplet_set_boolean_toggle(&pg->capp, "new_tabs_adjacent", FALSE);
+    capplet_set_boolean_toggle(&pg->capp, "ctrl_tab_shortcut", FALSE);
     capplet_set_radio(&pg->capp, "middle_click_tab", 0);
     profilegui_set_close_buttons_shading(pg);
     capplet_set_text_entry(&pg->capp, "ssh", NULL);
@@ -482,6 +489,9 @@ static void profilegui_fill_in_dialog(ProfileGUI * pg)
     capplet_set_spin_button(&pg->capp, "scrollback_lines", 1000);
     capplet_set_boolean_toggle(&pg->capp, "scroll_on_output", FALSE);
     capplet_set_boolean_toggle(&pg->capp, "scroll_on_keystroke", FALSE);
+    gboolean kinetic = capplet_set_boolean_toggle(&pg->capp,
+            "kinetic_scrolling", TRUE);
+    capplet_set_boolean_toggle(&pg->capp, "pixel_scrolling", kinetic);
     capplet_set_combo(&pg->capp, "backspace_binding",
             DEFAULT_BACKSPACE_BINDING);
     capplet_set_combo(&pg->capp, "delete_binding",
